import {CommonModule, isPlatformBrowser} from '@angular/common';
import {
  ChangeDetectorRef,
  Component,
  Directive,
  ElementRef,
  EventEmitter,
  Inject,
  Injectable,
  Input,
  NgModule,
  Output,
  PLATFORM_ID
} from '@angular/core';
import {Observable, Subject} from 'rxjs';
import {filter, finalize, take, takeUntil} from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InViewportComponent {
  /**
   * @param {?} changeDetectorRef
   */
  constructor(changeDetectorRef) {
    this.changeDetectorRef = changeDetectorRef;
  }

  /**
   * @param {?} intersectionObserverEntry
   * @return {?}
   */
  onViewportChange(intersectionObserverEntry) {
    if (intersectionObserverEntry.isIntersecting) {
      this.changeDetectorRef.reattach();
    } else {
      this.changeDetectorRef.detach();
    }
  }
}
InViewportComponent.decorators = [
  {
    type: Component, args: [{
      selector: 'aiut-in-viewport',
      template: "<div (inViewport)=\"onViewportChange($event)\" aiutInViewport>\r\n  <ng-content></ng-content>\r\n</div>\r\n"
    }]
  }
];
/** @nocollapse */
InViewportComponent.ctorParameters = () => [
  {type: ChangeDetectorRef}
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ViewportService {
  constructor() {
    this.options = {
      rootMargin: '0px 0px 0px 0px',
      threshold: [0.5],
    };
    this.callback$ = new Subject();
    this.observer = new IntersectionObserver(this.handler.bind(this), this.options);
  }

  /**
   * @param {?} element
   * @return {?}
   */
  observe(element) {
    this.observer.observe(element);
    return this.callback$.asObservable().pipe(filter((/**
     * @param {?} entry
     * @return {?}
     */
      (entry) => entry.target === element)), finalize((/**
     * @return {?}
     */
      () => this.observer.unobserve(element))));
  }

  /**
   * @private
   * @param {?} entries
   * @return {?}
   */
  handler(entries) {
    entries.forEach((/**
     * @param {?} entry
     * @return {?}
     */
      entry => this.callback$.next(entry)));
  }
}
ViewportService.decorators = [
  {type: Injectable}
];
/** @nocollapse */
ViewportService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// create a symbol identify the observable I add to
// the component so it doesn't conflict with anything.
// I need this so I'm able to add the desired behaviour to the component.
/** @type {?} */
const destroy$ = Symbol('destroy$');
/**
 * An operator that takes until destroy it takes a components this a parameter
 * returns a pipeable RxJS operator.
 * @type {?}
 */
const untilDestroy = (/**
 * @template T
 * @param {?} component
 * @return {?}
 */
  (component) => {
  if (component[destroy$] === undefined) {
    // only hookup each component once.
    addDestroyObservableToComponent(component);
  }
  // pipe in the takeUntil destroy$ and return the source unaltered
  return takeUntil(component[destroy$]);
});
/**
 * \@internal
 * @param {?} component
 * @return {?}
 */
function addDestroyObservableToComponent(component) {
  component[destroy$] = new Observable((/**
   * @param {?} observer
   * @return {?}
   */
    observer => {
    // keep track of the original destroy function,
    // the user might do something in there
    /** @type {?} */
    const orignalDestroy = component.ngOnDestroy;
    if (orignalDestroy == null) {
      // Angular does not support dynamic added destroy methods
      // so make sure there is one.
      throw new Error('untilDestroy operator needs the component to have an ngOnDestroy method');
    }
    // replace the ngOndestroy
    component.ngOnDestroy = (/**
     * @return {?}
     */
      () => {
      // fire off the destroy observable
      observer.next();
      // complete the observable
      observer.complete();
      // and at last, call the original destroy
      orignalDestroy.call(component);
    });
    // return cleanup function.
    return (/**
     * @param {?} _
     * @return {?}
     */
      (_) => (component[destroy$] = undefined));
  }));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InViewportDirective {
  /**
   * @param {?} elementRef
   * @param {?} viewportService
   * @param {?} platformId
   */
  constructor(elementRef, viewportService, platformId) {
    this.elementRef = elementRef;
    this.viewportService = viewportService;
    this.platformId = platformId;
    this.preRender = true;
    this.oneTime = false;
    this.inViewport = new EventEmitter();
  }

  /**
   * @return {?}
   */
  ngOnInit() {
    if (isPlatformBrowser(this.platformId)) {
      if (this.oneTime) {
        this.viewportService
          .observe(this.elementRef.nativeElement)
          .pipe(untilDestroy(this), filter((/**
           * @param {?} entry
           * @return {?}
           */
            entry => entry.intersectionRatio >= 0.5)), take(1))
          .subscribe((/**
           * @param {?} entry
           * @return {?}
           */
            (entry) => {
            this.inViewport.emit(entry);
          }));
      } else {
        this.viewportService
          .observe(this.elementRef.nativeElement)
          .pipe(untilDestroy(this))
          .subscribe((/**
           * @param {?} entry
           * @return {?}
           */
            (entry) => {
            this.inViewport.emit(entry);
          }));
      }
    } else {
      if (this.preRender) {
        this.inViewport.emit({isIntersecting: true, intersectionRatio: 1});
      }
    }
  }

  /**
   * @return {?}
   */
  ngOnDestroy() {
  }
}
InViewportDirective.decorators = [
  {
    type: Directive, args: [{
      selector: '[aiutInViewport]',
    },]
  }
];
/** @nocollapse */
InViewportDirective.ctorParameters = () => [
  {type: ElementRef},
  {type: ViewportService},
  {type: Object, decorators: [{type: Inject, args: [PLATFORM_ID,]}]}
];
InViewportDirective.propDecorators = {
  preRender: [{type: Input}],
  oneTime: [{type: Input}],
  inViewport: [{type: Output}]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AitpUtilsModule {
}
AitpUtilsModule.decorators = [
  {
    type: NgModule, args: [{
      imports: [CommonModule],
      declarations: [InViewportDirective, InViewportComponent],
      providers: [ViewportService],
      exports: [InViewportDirective, InViewportComponent]
    },]
  }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export {
  AitpUtilsModule,
  InViewportComponent,
  InViewportDirective,
  addDestroyObservableToComponent,
  destroy$,
  untilDestroy,
  ViewportService,
  InViewportComponent as ɵb,
  InViewportDirective as ɵa,
  ViewportService as ɵc
};

//# sourceMappingURL=aitp-utils.js.map
